#include <QTest>
#include "modules/value-editor/abstractkeyfactory.h"
#include "modules/value-editor/keymodel.h"


class FakeKeyModel : public ValueEditor::Model
{
public:
    QString getKeyName() override
    {
        return QString("fake");
    }

    QString getKeyTitle() override
    {
        return QString("fake");
    }

    QString getType() override
    {
        return QString("fake");
    }

    long long getTTL() override
    {
        return -1;
    }

    QStringList getColumnNames() override
    {
        return QStringList() << "value";
    }

    QHash<int, QByteArray> getRoles() override
    {
        QHash<int, QByteArray> result;
        result[Qt::UserRole + 1] = "value";
        return result;
    }

    QVariant getData(int rowIndex, int) override
    {
        if (rowIndex != 0)
            return QVariant();

        return QVariant(QString("fake value"));
    }

    void setKeyName(const QByteArray &) override
    {
    }

    void setTTL(const long long) override
    {
    }

    void removeKey() override
    {
    }

    void addRow(const QVariantMap &) override
    {
    }

    void updateRow(int rowIndex, const QVariantMap &) override
    {
    }

    unsigned long rowsCount() override
    {
        return 1;
    }

    void loadRows(unsigned long rowStart, unsigned long count, std::function<void (const QString &)> callback) override
    {
    }

    void clearRowCache() override
    {
    }

    void removeRow(int) override
    {
    }

    bool isRowLoaded(int) override
    {
        return true;
    }

    bool isMultiRow() const override
    {
        return true;
    }

    QSharedPointer<ValueEditor::ModelSignals> getConnector() const override
    {
        return QSharedPointer<ValueEditor::ModelSignals>();
    }

    QSharedPointer<RedisClient::Connection> getConnection() const override
    {
        return QSharedPointer<RedisClient::Connection>();
    }

    unsigned int dbIndex() const override
    {
        return 0u;
    }
};


class FakeKeyFactory : public ValueEditor::AbstractKeyFactory
{
public:
    void loadKey(QSharedPointer<RedisClient::Connection> connection,
                 QByteArray keyFullPath, int dbIndex,
                 std::function<void(QSharedPointer<ValueEditor::Model>, const QString&)> callback) override
    {
        loadKeyCalled++;
        QVERIFY(connection.isNull() == false);
        QVERIFY(keyFullPath.isEmpty() == false);
        QVERIFY(dbIndex >= 0);
        callback(QSharedPointer<ValueEditor::Model>(new FakeKeyModel()), QString());
    }

    void addKey(QSharedPointer<RedisClient::Connection> connection,
                 QByteArray keyFullPath, int dbIndex, QString type,
                 const QVariantMap &row) override
    {
        addKeyCalled++;
        QVERIFY(connection.isNull() == false);
        QVERIFY(keyFullPath.isEmpty() == false);
        QVERIFY(dbIndex >= 0);
        QVERIFY(type.isEmpty() == false);
        QVERIFY(row.isEmpty() == false);
    }

    uint loadKeyCalled;
    uint addKeyCalled;
};
